#!/bin/sh

check_feedback()
{
	if [ $? -gt 0 ]; then
		exit 1
	fi
}

cd /tmp
reboot=1
echo "*               Checking File integrity                 *"
echo "*********************************************************"
tar -xvf /tmp/upload.bin busybox
chmod 755 /tmp/busybox

checkmtd=1
/tmp/busybox nandwrite --show_info | grep "ubi.mtd=add"
checkmtd=$?
if [ "$checkmtd" == "1" ]; then
ubimtd=$(cat /proc/mtd | grep "add" | sed 's/mtd//g' | cut -d ":" -f 1)
else
ubimtd=$(cat /proc/mtd | grep "lnx" | sed 's/mtd//g' | cut -d ":" -f 1)
fi
ubitmp="/tmp/mtdtmp"
echo "*********************************************************"
echo "*               mounting /dev/mtd$ubimtd partion        *"
echo "*********************************************************"
/bin/grep "mtd$ubimtd" /proc/mtd

/usr/sbin/ubiattach /dev/ubi_ctrl -m $ubimtd -d 1
check_feedback
/bin/mkdir -p $ubitmp
check_feedback
/bin/mount -t ubifs ubi1_0 $ubitmp
check_feedback
/bin/rm -Rf $ubitmp/*

tar -xvf /tmp/upload.bin -C $ubitmp updateam.bin fsize.t
if test -e '/tmp/mtdtmp/fsize.t' ; then
	md5=$(grep "MD5" $ubitmp/fsize.t | sed 's/MD5=//g')
else
  md5="md5"
fi
if test -e '/tmp/mtdtmp/updateam.bin' ; then
	MD5_chk=$(md5sum /tmp/mtdtmp/updateam.bin | sed 's/\/tmp\/mtdtmp\/updateam.bin//g')
else
	MD5_chk="MD5_chk"
fi

if [ "$MD5_chk" = "$md5" ] ; then
	echo "MD5_chk correct.......Proceed with upgrade"
	reboot=0
else
	echo "MD5_chk DO NOT MATCH $md5 != $MD5_chk...DISCARD upload, REBOOT in 3 seconds"
	reboot=1
#	sleep 3
fi

if [ "$reboot" = "0" ] ; then

echo "*********************************************************"
echo "*               Copy nvram data to mtd6 start           *"
echo "*********************************************************"
#copy all files to /tmp/ and add --myboundary to file end
echo --myboundary > /tmp/_nvram.now
cat /www/nvram.now >>/tmp/_nvram.now
echo --myboundary >> /tmp/_nvram.now

echo --myboundary > /tmp/_nvram.roi
cat /www/nvram.roi >> /tmp/_nvram.roi
echo --myboundary >> /tmp/_nvram.roi

echo --myboundary > /tmp/_nvram.mac
cat /etc/nvram.mac >> /tmp/_nvram.mac
echo --myboundary >> /tmp/_nvram.mac

echo --myboundary > /tmp/_nvram.ip
cat /etc/nvram.ip >> /tmp/_nvram.ip
echo --myboundary >> /tmp/_nvram.ip

echo --myboundary > /tmp/_nvram.factory
cat /etc/nvram.factory >> /tmp/_nvram.factory
echo --myboundary >> /tmp/_nvram.factory

echo --myboundary > /tmp/_nvram.patrol1
cat /www/nvram.patrol1 >>/tmp/_nvram.patrol1
echo --myboundary >> /tmp/_nvram.patrol1

echo --myboundary > /tmp/_nvram.patrol2
cat /www/nvram.patrol2 >>/tmp/_nvram.patrol2
echo --myboundary >> /tmp/_nvram.patrol2

echo --myboundary > /tmp/_nvram.patrol3
cat /www/nvram.patrol3 >>/tmp/_nvram.patrol3
echo --myboundary >> /tmp/_nvram.patrol3

echo --myboundary > /tmp/_nvram.patrol4
cat /www/nvram.patrol4 >>/tmp/_nvram.patrol4
echo --myboundary >> /tmp/_nvram.patrol4

#flash erase (mtd6 is configure for 16MB)
/tmp/busybox flash_eraseall /dev/mtd6
/tmp/busybox nandwrite -p -s 0x000000 /dev/mtd6 /tmp/_nvram.now
/tmp/busybox nandwrite -p -s 0x100000 /dev/mtd6 /tmp/_nvram.roi
/tmp/busybox nandwrite -p -s 0x200000 /dev/mtd6 /tmp/_nvram.mac
/tmp/busybox nandwrite -p -s 0x280000 /dev/mtd6 /tmp/_nvram.ip
/tmp/busybox nandwrite -p -s 0x300000 /dev/mtd6 /tmp/_nvram.factory
/tmp/busybox nandwrite -p -s 0x400000 /dev/mtd6 /tmp/_nvram.patrol1
/tmp/busybox nandwrite -p -s 0x500000 /dev/mtd6 /tmp/_nvram.patrol2
/tmp/busybox nandwrite -p -s 0x600000 /dev/mtd6 /tmp/_nvram.patrol3
/tmp/busybox nandwrite -p -s 0x700000 /dev/mtd6 /tmp/_nvram.patrol4

rm -f /tmp/_nvram.now
rm -f /tmp/_nvram.roi
rm -f /tmp/_nvram.mac
rm -f /tmp/_nvram.ip
rm -f /tmp/_nvram.factory
rm -f /tmp/_nvram.patrol1
rm -f /tmp/_nvram.patrol2
rm -f /tmp/_nvram.patrol3
rm -f /tmp/_nvram.patrol4

echo "*********************************************************"
echo "*               Copy nvram data to mtd6 end             *"
echo "*********************************************************"

echo "*********************************************************"
echo "*           Extract & Checking All Files                *"
echo "*********************************************************"
echo "tar -xvf /tmp/mtdtmp/updateam.bin -C /tmp"
tar -xvf /tmp/mtdtmp/updateam.bin -C /tmp mtd0.bin mtd1.bin mtd2.bin mtd3.bin
totalcnt=0
if test -e '/tmp/mtd0.bin' ; then
totalcnt=$(($totalcnt+1))
fi
if test -e '/tmp/mtd1.bin' ; then
totalcnt=$(($totalcnt+1))
fi
if test -e '/tmp/mtd2.bin' ; then
totalcnt=$(($totalcnt+1))
fi
if test -e '/tmp/mtd3.bin' ; then
totalcnt=$(($totalcnt+1))
fi

if [ "$totalcnt" == "4" ]; then
	echo "ALL $totalcnt File extracted, Start Update process"
else
	echo "*********************************************************"
	echo "*           !!!!Only $totalcnt files extracted          *"
	echo "*           !!!!Missing files, ABORT UPDATE             *"
	echo "*********************************************************"
	reboot
	exit
fi

echo "*********************************************************"
echo "*               Update mtd 0~4 Partitions end           *"
echo "*********************************************************"
	/tmp/busybox flash_eraseall /dev/mtd0
	/tmp/busybox nandwrite -p /dev/mtd0 mtd0.bin
	sync
	/tmp/busybox flash_eraseall /dev/mtd1
	/tmp/busybox nandwrite -p /dev/mtd1 mtd1.bin
	sync
	/tmp/busybox flash_eraseall /dev/mtd2
	/tmp/busybox nandwrite -p /dev/mtd2 mtd2.bin
	sync
	/tmp/busybox flash_eraseall /dev/mtd3
	/tmp/busybox nandwrite -p /dev/mtd3 mtd3.bin
	sync
echo "*********************************************************"
echo "*               Update mtd 0~4 Partitions end           *"
echo "*********************************************************"

cd /tmp
#mv /tmp/updateam.bin /home/lilin/
rm -f /tmp/mtd*.bin
rm -f /tmp/upload.bin

echo "*********************************************************"
echo "*           Extract & Checking All Files                *"
echo "*********************************************************"
echo "tar -xvf $ubitmp/updateam.bin -C /tmp Image"
tar -xvf $ubitmp/updateam.bin -C /tmp Image
totalcnt=0

echo "*********************************************************"
echo "*               Update kernel Image start to /dev/mtd5  *"
echo "*********************************************************"

	echo "Start flash write kernel"
	/tmp/busybox flash_eraseall /dev/mtd5
	/tmp/busybox nandwrite -p -K -L 0xC0208000 --pri_addr=0x01F20000 /dev/mtd5 Image
	sync

echo "*********************************************************"
echo "*               Update kernel Image end                 *"
echo "*********************************************************"

echo "*********************************************************"
echo "*               Erasing rootfs start  /dev/mtd$ubimtd   *"
echo "*********************************************************"

rm -f /tmp/Image
echo "tar -xvf $ubitmp/updateam.bin -C /tmp ubifs"
tar -xvf $ubitmp/updateam.bin -C /tmp ubifs
sync

/bin/umount $ubitmp
/usr/sbin/ubidetach /dev/ubi_ctrl -d 1

if [ "$checkmtd" == "1" ]; then
	/tmp/busybox flash_eraseall /dev/mtd$ubimtd
	/tmp/busybox nandwrite -p -Y /dev/mtd$ubimtd ubifs
	/tmp/busybox nandwrite -C "console=ttyS0 ubi.mtd=add root=ubi0:rootfs rw rootfstype=ubifs init=/linuxrc"
else
	/tmp/busybox flash_eraseall /dev/mtd$ubimtd
	/tmp/busybox nandwrite -p -W /dev/mtd$ubimtd ubifs
	/tmp/busybox nandwrite -C "console=ttyS0 ubi.mtd=lnx root=ubi0:rootfs rw rootfstype=ubifs init=/linuxrc"
fi

fi
sleep 1
sync

echo "*********************************************************"
echo "*               Erasing rootfs end                      *"
echo "*********************************************************"


echo "*********************************************************"
echo "*               Resetting system start reset            *"
echo "*********************************************************"
reboot
echo "*********************************************************"
echo "*               Resetting system end reset              *"
echo "*********************************************************"
